#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <regex.h>
#include <gtk/gtk.h>
#include "resource.h"

/*
Changelog:
Interface creation
*/

/* Holds widgets for g_signal_connect */
typedef struct {
	GtkWidget *word, *output, *statusbar, *window;
} Widgets;

/* Match action */
int match_word(char *phrase, char *word, GtkWidget *widget, Widgets *info) {
	regex_t haystack;
	regmatch_t search_match[100];
	int status;
	GtkTextBuffer *buffer;
	/* Compile regex */
	if(regcomp(&haystack, phrase, REG_NEWLINE|REG_ICASE) == 0) {
		/* Check for a match */
		if(regexec(&haystack, word, 1, search_match, 0) == 0) {
			/* Insert match */
			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(info->output));
			gtk_text_buffer_insert_at_cursor(buffer, word + search_match[0].rm_so, -1);
			status = 1;
		} else {
			/* No match */
			status = 0;
		}
	} else {
		/* Broken regex */
		status = -100;
	}
	regfree(&haystack);
	return status;
}

/* Search action */
void search_sub(GtkWidget *widget, Widgets *info) {
	const gchar *word;
	GtkTextBuffer *buffer;
	char phrase[255];
	word = gtk_entry_get_text(GTK_ENTRY(info->word));
	if(strlen(word) == 0) {
		/* Empty search */
		gtk_statusbar_push(GTK_STATUSBAR(info->statusbar), 0, "No match searched");
	} else if(strlen(word) > 254) {
		/* Large search */
		gtk_statusbar_push(GTK_STATUSBAR(info->statusbar), 0, "Search too long");
	} else {
		/* Clear output */
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(info->output));
		gtk_text_buffer_set_text(buffer, "", -1);
		sprintf(phrase, "^%s$", word);
		FILE *contents;
		char line[255];
		int match = 0;
		char match_str[255];
		char error[255];
		/* Open word list */
		if((contents = fopen(WORDLIST, "r")) == NULL) {
			sprintf(error, "Cannot open word list %s", WORDLIST);
			gtk_statusbar_push(GTK_STATUSBAR(info->statusbar), 0, error);
		} else {
			/* Read line by line */
			while(fgets(line, sizeof(line), contents) != NULL && match > -1) {
				/* Narrow to only matching string length */
				if(strlen(line) == strlen(phrase) - 1) {
					match += match_word(phrase, line, widget, info);
				}
			}
			if(match > -1) {
				/* Display match count */
				sprintf(match_str, "%d matches", match);
				gtk_statusbar_push(GTK_STATUSBAR(info->statusbar), 0, match_str);
			} else {
				/* Broken pattern */
				sprintf(error, "Error matching pattern %s", word);
				gtk_statusbar_push(GTK_STATUSBAR(info->statusbar), 0, error);
			}
			fclose(contents);
		}
	}
}

/* The main code execution and display */
int main(int argc, char *argv[]) {
	/* Variables */
	GtkWidget *vbox;
	GtkWidget *button;
	GtkWidget *scrollarea;	
	GtkWidget *directions;
	Widgets *info = (Widgets*) g_malloc(sizeof(Widgets));
	gtk_init(&argc, &argv);

	/* Window attributes */
	info->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(info->window), GTK_WIN_POS_MOUSE);
	gtk_widget_set_size_request(GTK_WIDGET(info->window), 270, 280);
	gtk_window_set_title(GTK_WINDOW(info->window), "Guess");
	g_signal_connect(G_OBJECT(info->window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect_swapped(G_OBJECT(info->window), "delete_event", G_CALLBACK(gtk_widget_destroy), G_OBJECT(info->window));
	gtk_container_set_border_width(GTK_CONTAINER(info->window), 3);

	/* Vbox for packing */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(info->window), vbox);
	gtk_widget_show(vbox);

	/* Directions section */
	directions = gtk_label_new("Search: Use a . for each unknown letter");
	gtk_misc_set_alignment(GTK_MISC(directions), 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox), directions, TRUE, TRUE, 0);
	gtk_widget_show(directions);
	
	/* Word section */
	info->word = gtk_entry_new();
	g_signal_connect(G_OBJECT(info->word), "activate", G_CALLBACK(search_sub), (gpointer) info);
	gtk_box_pack_start(GTK_BOX(vbox), info->word, TRUE, TRUE, 0);
	gtk_widget_show(info->word);

	/* Search button */
	button = gtk_button_new_with_label("Search");
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(search_sub), (gpointer) info);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 5);
	gtk_widget_show(button);

	/* Scroll Container */
	scrollarea = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollarea), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollarea), GTK_SHADOW_IN);
	gtk_widget_set_size_request(scrollarea, 0, 170);
	gtk_box_pack_start(GTK_BOX(vbox), scrollarea, TRUE, TRUE, 0);
	gtk_widget_show(scrollarea);

	/* Output display */
	info->output = gtk_text_view_new();
	gtk_container_add(GTK_CONTAINER(scrollarea), info->output);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(info->output), 0);
	gtk_widget_show(info->output);

	/* Status bar */
	info->statusbar = gtk_statusbar_new();
	gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(info->statusbar), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), info->statusbar, TRUE, TRUE, 0);
	gtk_widget_show(info->statusbar);

	/* Show windows */
	gtk_widget_show(info->window);
	gtk_main();

	return 0;
}
